package net.ko.mapping;

import java.util.ArrayList;
import java.util.regex.Matcher;

import javax.servlet.http.HttpServletRequest;

import net.ko.debug.KDebugClient;
import net.ko.design.KCssTransition;
import net.ko.utils.KHash;
import net.ko.utils.KString;
import net.ko.utils.KStrings;
import net.ko.utils.KStrings.KGlueMode;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class KAjaxJs implements IAjaxObject {
	private String triggerSelector;
	private String triggerContext = "document";
	private String triggerEvent;
	private ArrayList<IAjaxObject> childs;
	private HttpServletRequest request;
	private int keyCode = -1;
	protected boolean isInRootFolder;
	protected String clientDebugString = "";
	protected String key;
	protected boolean breakPoint;

	public KAjaxJs() {
		childs = new ArrayList<>();
	}

	public KAjaxJs(String triggerSelector, String triggerContext, String triggerEvent) {
		this();
		this.triggerSelector = triggerSelector;
		this.triggerContext = triggerContext;
		setTriggerEvent(triggerEvent);
	}

	@Override
	public boolean isValid() {
		boolean result = true;
		result = result && (KString.isNotNull(triggerSelector));
		return result;
	}

	private boolean isKeyEvent() {
		return (triggerEvent.startsWith("onkey") || triggerEvent.startsWith("key")) && keyCode != -1;
	}

	@Override
	public String createFunction() {
		String result = "function(e){var target=$gte(e);$sp(e);";
		String startKc = "";
		String endKc = "";
		if (isKeyEvent()) {
			startKc = "if($gkc(e)==" + keyCode + "){";
			endKc = "return false;}";
		}
		result += startKc;
		for (IAjaxObject obj : childs) {
			if (obj instanceof KAjaxSet)
				((KAjaxSet) obj).setRequest(request);
			result += obj.createFunction();
		}
		result += endKc;
		return result + "}";
	}

	public String getExecuteString() {
		String result = "";
		for (IAjaxObject obj : childs) {
			if (obj instanceof KAjaxSet)
				((KAjaxSet) obj).setRequest(request);
			result += obj.createFunction();
		}
		return result;
	}

	@Override
	public String toString() {
		String result = "";
		if (KString.isNotNull(triggerSelector)) {
			if (triggerSelector.equalsIgnoreCase("body") && triggerEvent.contains("load"))
				result = "Forms.DOM.onReady(function(){" + getExecuteString() + "});\n";
			else {
				if (keyCode < 1)
					result = "new Forms.Elements('" + triggerSelector + "'," + triggerContext + ").addEvent('" + triggerEvent + "'," + createFunction() + ",false);\n";
				else
					result = "new Forms.Elements('" + triggerSelector + "'," + triggerContext + ").addEvent('" + triggerEvent + "'," + createFunction() + ",false," + keyCode + ");\n";
			}
		}
		result = replaceJsExpressions(result);
		return result;
	}

	public String getTriggerEvent() {
		return triggerEvent;
	}

	protected void setTriggerEvent(String triggerEvent) {
		if (KString.isNull(triggerEvent))
			triggerEvent = "click";
		if (triggerEvent.startsWith("on"))
			triggerEvent = triggerEvent.substring(2);
		triggerEvent = triggerEvent.toLowerCase();
		this.triggerEvent = triggerEvent;
	}

	public static ArrayList<IAjaxObject> getChilds(Node item, boolean isChild, boolean isInRootFolder) {
		return getChilds(item, isChild, isInRootFolder, false);
	}

	public static ArrayList<IAjaxObject> getChilds(Node item, boolean isChild, boolean isInRootFolder, boolean force) {
		ArrayList<IAjaxObject> result = new ArrayList<>();
		NodeList cNodes = item.getChildNodes();
		for (int i = 0; i < cNodes.getLength(); i++) {
			KAjaxObject obj = null;
			switch (cNodes.item(i).getNodeName()) {
			case "include":
				obj = new KAjaxInclude();
				obj.initFromXml(cNodes.item(i));
				break;
			case "function":
				obj = new KAjaxFunction();
				obj.initFromXml(cNodes.item(i));
				break;
			case "message":
				obj = new KAjaxMessage();
				obj.initFromXml(cNodes.item(i));
				break;
			case "showHide":
				obj = new KAjaxShowHide();
				obj.initFromXml(cNodes.item(i));
				break;
			case "selector":
				obj = new KAjaxSelector();
				obj.initFromXml(cNodes.item(i));
				break;
			case "set":
				obj = new KAjaxSet();
				obj.initFromXml(cNodes.item(i));
				break;
			case "fireEvent":
				obj = new KAjaxEvent();
				obj.initFromXml(cNodes.item(i));
				break;
			case "messageDialog":
				obj = new KAjaxMessageDialog();
				obj.initFromXml(cNodes.item(i));
				break;
			case "button":
				obj = new KAjaxDialogButton();
				obj.initFromXml(cNodes.item(i));
				break;
			case "includeDialog":
				obj = new KAjaxIncludeDialog();
				obj.initFromXml(cNodes.item(i));
				break;
			case "submitForm":
				obj = new KAjaxSubmitForm();
				obj.initFromXml(cNodes.item(i));
				break;
			case "refreshControl":
				obj = new KAjaxRefreshControl();
				obj.initFromXml(cNodes.item(i));
				break;
			case "includeForm":
				obj = new KAjaxIncludeForm();
				obj.initFromXml(cNodes.item(i));
				break;
			case "accordion":
				obj = new KAjaxAccordion();
				obj.initFromXml(cNodes.item(i));
				break;
			case "refreshFormValues":
				obj = new KAjaxRefreshFormValues();
				obj.initFromXml(cNodes.item(i));
				break;

			case "deleteOne":
				obj = new KAjaxDeleteOne();
				obj.initFromXml(cNodes.item(i));
				break;

			case "updateOne":
				obj = new KAjaxUpdateOne();
				obj.initFromXml(cNodes.item(i));
				break;

			case "transition":
				obj = new KCssTransition();
				obj.initFromXml(cNodes.item(i));
				break;

			default:
				break;
			}
			if (obj != null && (obj.isValid() || force)) {
				obj.setInRootFolder(isInRootFolder);
				obj.setChild(isChild);
				obj.setKey(item.hashCode() + "-" + result.size());
				result.add(obj);
			}
		}
		return result;
	}

	@Override
	public void initFromXml(Node item) {
		Element e = (Element) item;
		setTriggerEvent(e.getAttribute("triggerEvent"));
		String triggerSelector = e.getAttribute("triggerSelector");
		if (triggerSelector != null && !"".equals(triggerSelector))
			this.triggerSelector = triggerSelector;
		String triggerContext = e.getAttribute("triggerContext");
		if (triggerContext != null && !"".equals(triggerContext))
			this.triggerContext = triggerContext;
		String strKeyCode = e.getAttribute("keyCode");
		if (strKeyCode != null && !"".equals(strKeyCode)) {
			try {
				this.keyCode = Integer.valueOf(strKeyCode);
			} catch (Exception ex) {
				keyCode = -1;
			}
		}
		childs = getChilds(item, false, isInRootFolder);
		boolean breakPoint = KString.isBooleanTrue(e.getAttribute("break"));
		setBreakPoint(breakPoint);
	}

	@Override
	public String execute(HttpServletRequest request) {
		this.request = request;
		return toString();
	}

	public ArrayList<IAjaxObject> getChilds() {
		return childs;
	}

	@Override
	public String getDisplayCaption() {
		return triggerSelector + ".on" + KString.capitalizeFirstLetter(getTriggerEvent());
	}

	@Override
	public String display(int margin) {
		String structure = getDisplayStructure();
		structure = structure.replace("{caption}", getDisplayCaption());
		return "<div style='margin-left:" + (20 * margin) + "px;' class='" + getClass().getSimpleName() + "'>" + structure + "</div>";
	}

	@Override
	public String getUniqueIdHash(int niveau) {
		return KHash.getMD5(niveau + getDisplayCaption());
	}

	@Override
	public String getDisplayStructure() {
		return "{caption}";
	}

	@Override
	public boolean hasChilds() {
		return true;
	}

	@Override
	public String replaceJsExpressions(String input) {
		String output = input;
		if (input.contains("{js")) {
			output = output.replaceAll("\\{js\\:(.+?)\\}", Matcher.quoteReplacement("'+") + "$1" + Matcher.quoteReplacement("+'"));
			output = output.replaceAll("\\{jsv\\:(.+?)\\}", Matcher.quoteReplacement("'+$$('") + "$1" + Matcher.quoteReplacement("')+'"));
		}
		return output;
	}

	@Override
	public String executeByName() {
		return toString();
	}

	public boolean isInRootFolder() {
		return isInRootFolder;
	}

	public void setInRootFolder(boolean isInRootFolder) {
		this.isInRootFolder = isInRootFolder;
	}

	@Override
	public String getDisplayDebugClientInfo() {
		return triggerSelector + ".on" + KString.capitalizeFirstLetter(getTriggerEvent());
	}

	@Override
	public String getDebugClientInfo(int index) {
		String result = "";
		String id = getDebugClientId();
		if (KDebugClient.hasOption(getName())) {
			if (!"".equals(id)) {
				result = "$infoBulle('" + id + "','" + getDisplayDebugClientInfo() + "',{" + getClientDebugTitle() + "},'" + getName() + "',true,'" + getKey() + "');\n";
			}
		}
		result += getChildsDebugClientInfo();
		return result;

	}

	@Override
	public String getDebugClientId() {
		return triggerSelector;
	}

	@Override
	public String getName() {
		return "JS";
	}

	@Override
	public String getClientDebugString() {
		return clientDebugString;
	}

	@Override
	public String getChildsDebugClientInfo() {
		String clientDebugString = "";
		String tmp = "";
		int i = 0;
		for (IAjaxObject child : childs) {
			tmp += child.getDebugClientInfo(i);
			i++;
		}
		String parent = getDebugClientId().replaceAll("\\W", "") + "-infoBulle-" + getName() + getKey();
		clientDebugString += tmp.replace("%visibility%", "false").replace("%parent%", ",'" + parent + "'");
		clientDebugString += "$addUEvt($('" + parent + "'),'click',function(e){event=Forms.Private.getEvent(e);if(event.ctrlKey==1){ Forms.Utils.stopPropagation(event);event.target.areChildsVisible=event.target.areChildsVisible?false:true;" + tmp.replace("%visibility%", "event.target.areChildsVisible").replace("%parent%", "") + "}});";
		return clientDebugString;
	}

	@Override
	public String getClientDebugTitle() {
		return getTitles().implode_param(",", "\"", ":", KGlueMode.KEY_AND_VALUE, "\"", false);
	}

	public KStrings getTitles() {
		KStrings titles = new KStrings();
		if (request != null)
			titles.put("request", request.getRequestURI());
		if (keyCode != -1)
			titles.put("keyCode", keyCode + "");
		return titles;
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	@Override
	public void setBreaked(boolean breaked) {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean isBreakPoint() {
		return breakPoint;
	}

	public void setBreakPoint(boolean breakPoint) {
		this.breakPoint = breakPoint;
		if (breakPoint) {
			for (IAjaxObject child : childs) {
				child.setBreakPoint(breakPoint);
			}
		}
	}
}
