package net.ko.mapping;

import java.util.logging.Level;

import net.ko.framework.Ko;
import net.ko.utils.KXmlFile;

import org.w3c.dom.NodeList;

public class KXmlMappings {
	private KXmlFile xmlFile;
	private static KXmlMappings instance;
	private KFilterMappings mappings;
	private KFilterMappings filters;
	private KAjaxIncludes ajaxIncludes;
	private boolean isLoaded;

	private KXmlMappings() {
		mappings = new KFilterMappings();
		filters = new KFilterMappings();
		addActionsFilter();
		ajaxIncludes = new KAjaxIncludes();
		isLoaded = false;
	}

	public static void stop() {
		KXmlMappings.instance = null;
	}

	public static KXmlMappings getInstance() {
		if (KXmlMappings.instance == null)
			KXmlMappings.instance = new KXmlMappings();
		return instance;
	}

	public void loadFromFile(String fileName, boolean force) {
		if (!isLoaded || force) {
			xmlFile = new KXmlFile();
			try {
				xmlFile.loadFromFile(fileName);
				initMappings();
				isLoaded = true;
			} catch (Exception e) {
				Ko.klogger().log(Level.SEVERE, "Impossible de terminer la lecture du fichier de configuration " + fileName, e);
				isLoaded = false;
			}
		}
	}

	private void initMappings() {
		NodeList nodes = xmlFile.getChildNodes();
		for (int i = 0; i < nodes.getLength(); i++) {
			if (nodes.item(i).getNodeName().equals("mappings")) {
				NodeList cNodes = nodes.item(i).getChildNodes();
				for (int j = 0; j < cNodes.getLength(); j++) {
					if (cNodes.item(j).getNodeName().equals("mapping")) {
						KMapping mapping = new KMapping();
						mapping.initFromXml(cNodes.item(j));
						mappings.add(mapping);
					} else if (cNodes.item(j).getNodeName().equals("virtualMapping")) {
						KMapping mapping = new KVirtualMapping();
						mapping.initFromXml(cNodes.item(j));
						mappings.add(mapping);
					}
				}
			} else if (nodes.item(i).getNodeName().equals("ajax-includes")) {
				NodeList cNodes = nodes.item(i).getChildNodes();
				for (int j = 0; j < cNodes.getLength(); j++) {
					if (cNodes.item(j).getNodeName().equals("request")) {
						KAjaxRequest ajaxRequest = new KAjaxRequest();
						ajaxRequest.initFromXml(cNodes.item(j));
						ajaxIncludes.add(ajaxRequest);
					}
				}
			}
			else if (nodes.item(i).getNodeName().equals("filters")) {
				NodeList cNodes = nodes.item(i).getChildNodes();
				for (int j = 0; j < cNodes.getLength(); j++) {
					if (cNodes.item(j).getNodeName().equals("filter")) {
						KFilter filter = new KFilter();
						filter.initFromXml(cNodes.item(j));
						filters.add(filter);
					}
				}
			}
		}
	}

	private void addActionsFilter() {
		String actName = Ko.kactionsName();
		KFilter actionFilter = new KFilter("/{#(.+?)/(.+?)\\b(.*)#}/" + actName, "/$1/" + actName);
		actionFilter.addParameter("_action", 2);
		actionFilter.addParameter("_parameters", 3);
		filters.add(actionFilter);
	}

	public KFilterMappings getMappings() {
		return mappings;
	}

	public KFilterMappings getFilters() {
		return filters;
	}

	public KAjaxIncludes getAjaxIncludes() {
		return ajaxIncludes;
	}
}
